import { gql } from '@apollo/client/core'

import { FullUserFragmentDoc } from '@holi/api/graphql/graphql-codegen'
import { AuthenticatedUser, User } from '@holi/core/domain/shared/types'
import type { Paged } from '@holi/core/types'

export const sdgsQuery = gql`
  query SDGs {
    sdgs {
      data {
        id
        title
      }
    }
  }
`

export const skillsQuery = gql`
  query Skills {
    skills {
      data {
        id
        title
        slug
      }
    }
  }
`

export const topicsQuery = gql`
  query Topics {
    topics {
      data {
        id
        title
        slug
        isDiscoverSpacesDefault
      }
    }
  }
`
// This fragment is used in a lot of places throughout the app
// Do NOT add anymore fields here that are not required for HoliAvatar
export const UserAvatarFragment = gql`
  fragment UserAvatar on User {
    id
    avatar
    avatarBlurhash
    avatarLabel
    avatarDefaultColor
    firstName
    fullName
  }
`

export const FullUserAvatarFragment = gql`
  fragment FullUserAvatar on User {
    id
    avatar
    avatarBlurhash
    avatarLabel
    avatarDefaultColor
    firstName
    fullName
    role
  }
`

export const FullAuthenticatedUserFragment = gql`
  fragment FullAuthenticatedUser on AuthenticatedUser {
    id
    identity
    avatar
    avatarBlurhash
    avatarDefaultColor
    avatarLabel
    fullName
    firstName
    lastName
    aboutMe
    pronouns
    location
    role
    geolocationId
    engagementLevel
    interests {
      id
      title
      slug
    }
    sdgs {
      id
      title
      slug
    }
    skills {
      id
      title
      slug
    }
    connectionStatusToMyself
    trackingConsentAnalytics
    trackingConsentPersonalization
    isEmployee
  }
`
export const authenticatedUserV2Query = gql`
  ${FullAuthenticatedUserFragment}

  query AuthenticatedUser {
    authenticatedUserV2 {
      ...FullAuthenticatedUser
    }
  }
`

export type AuthenticatedUserV2Response = {
  authenticatedUserV2: AuthenticatedUser & {
    __typename: 'AuthenticatedUser'
  }
}

export const userByIdQuery = gql`
  ${FullUserFragmentDoc}

  query UserById($userId: UUID!) {
    userById(id: $userId) {
      ...FullUser
    }
  }
`

export interface UserByIdResponse {
  userById: User
}

export const userByNameQuery = gql`
  ${UserAvatarFragment}
  query UserByName($substring: String!, $limit: Int!, $offset: Int!) {
    userByName(substring: $substring, limit: $limit, offset: $offset) {
      data {
        ...UserAvatar
        identity
      }
      totalResults
    }
  }
`
export type UserByNameResponse = {
  userByName: Paged<User>
}

export const usersByChatIdentitiesQuery = gql`
  ${UserAvatarFragment}
  query UsersByChatIdentities($identities: [String!]!, $limit: Int!, $offset: Int!) {
    usersByChatIdentities(identities: $identities, limit: $limit, offset: $offset) {
      data {
        ...UserAvatar
        identity
        pronouns
      }
    }
  }
`

export type UsersByChatIdentitiesQueryVariables = {
  identities: string[]
  offset: number
  limit: number
}

export type UsersByChatIdentitiesResponse = {
  usersByChatIdentities: Paged<User>
}
